# Copyright (C) 2020  Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from fedora_messaging import message
from fedora_messaging.schema_utils import user_avatar_url


SCHEMA_URL = "http://fedoraproject.org/message-schema/"

MEETING = {
    "type": "object",
    "properties": {
        "meeting_id": {"type": "number"},
        "meeting_name": {"type": "string"},
        "meeting_manager": {"type": "array", "items": {"type": ["string", "null"]}},
        "meeting_date": {"type": "string"},
        "meeting_date_end": {"type": "string"},
        "meeting_time_start": {"type": "string"},
        "meeting_time_stop": {"type": "string"},
        "meeting_timezone": {"type": "string"},
        "meeting_information": {"type": ["string", "null"]},
        "meeting_location": {"type": ["string", "null"]},
        "calendar_name": {"type": "string"},
    },
    "required": [
        "meeting_id",
        "meeting_name",
        "meeting_manager",
        "meeting_date",
        "meeting_date_end",
        "meeting_time_start",
        "meeting_time_stop",
        "meeting_timezone",
        "meeting_information",
        "meeting_location",
        "calendar_name",
    ],
}

CALENDAR = {
    "type": "object",
    "properties": {
        "calendar_name": {"type": "string"},
        "calendar_contact": {"type": "string"},
        "calendar_description": {"type": ["string", "null"]},
        "calendar_editor_group": {"type": ["string", "null"]},
        "calendar_admin_group": {"type": ["string", "null"]},
        "calendar_status": {"type": "string"},
    },
    "required": [
        "calendar_name",
        "calendar_contact",
        "calendar_description",
        "calendar_editor_group",
        "calendar_admin_group",
        "calendar_status",
    ],
}


class FedocalMessage(message.Message):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by fedocal.
    """

    @property
    def app_name(self):
        return "fedocal"

    @property
    def app_icon(self):
        return "https://apps.fedoraproject.org/img/icons/fedocal.png"

    @property
    def agent(self):
        return self.body.get("agent")

    @property
    def agent_avatar(self):
        return user_avatar_url(self.agent)

    @property
    def usernames(self):
        if self.agent:
            return [self.agent]
        else:
            return []

    @property
    def url(self):
        try:
            return self.body["thing"]["url"]
        except KeyError:
            return None



# Copyright (C) 2020  Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import datetime

import dateutil.relativedelta

from .base import CALENDAR, FedocalMessage, MEETING, SCHEMA_URL


def _casual_timedelta_string(meeting):
    """Return a casual timedelta string.
    If a meeting starts in 2 hours, 15 minutes, and 32 seconds from now, then
    return just "in 2 hours".
    If a meeting starts in 7 minutes and 40 seconds from now, return just "in 7
    minutes".
    If a meeting starts 56 seconds from now, just return "right now".
    """

    now = datetime.datetime.utcnow()
    mdate = meeting["meeting_date"]
    mtime = meeting["meeting_time_start"]
    dt_string = "%s %s" % (mdate, mtime)
    meeting_dt = datetime.datetime.strptime(dt_string, "%Y-%m-%d %H:%M:%S")
    relative_td = dateutil.relativedelta.relativedelta(meeting_dt, now)

    denominations = ["years", "months", "days", "hours", "minutes"]
    for denomination in denominations:
        value = getattr(relative_td, denomination)
        if value:
            # If the value is only one, then strip off the plural suffix.
            if value == 1:
                denomination = denomination[:-1]
            return "in %i %s" % (value, denomination)

    return "right now"


class ReminderV1(FedocalMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by fedocal when a reminder is sent.
    """

    topic = "fedocal.meeting.reminder"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "Schema for messages sent when a reminder is sent",
        "type": "object",
        "properties": {"calendar": CALENDAR, "meeting": MEETING},
        "required": ["calendar", "meeting"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return (
            "Friendly reminder!  The '{meeting}' meeting from "
            "the '{calendar}' calendar starts {timestring}".format(
                meeting=self.body["meeting"]["meeting_name"],
                calendar=self.body["calendar"]["calendar_name"],
                timestring=_casual_timedelta_string(self.body["meeting"]),
            )
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return (
            "Remember about the meeting '{meeting}' from "
            "the '{calendar}' calendar".format(
                meeting=self.body["meeting"]["meeting_name"],
                calendar=self.body["calendar"]["calendar_name"],
            )
        )


class CalendarNewV1(FedocalMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by fedocal when a calendar is created.
    """

    topic = "fedocal.calendar.new"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "Schema for messages sent when a calendar is created",
        "type": "object",
        "properties": {"agent": {"type": "string"}, "calendar": CALENDAR},
        "required": ["calendar", "agent"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "{user} has created a new calendar {calendar}".format(
            user=self.body["agent"],
            calendar=self.body["calendar"]["calendar_name"],
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return str(self)


class CalendarUpdateV1(FedocalMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by fedocal when a calendar is updated.
    """

    topic = "fedocal.calendar.update"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "Schema for messages sent when a calendar is updated",
        "type": "object",
        "properties": {"agent": {"type": "string"}, "calendar": CALENDAR},
        "required": ["calendar", "agent"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "{user} has updated a calendar {calendar}".format(
            user=self.body["agent"],
            calendar=self.body["calendar"]["calendar_name"],
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return str(self)


class CalendarUploadV1(FedocalMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by fedocal when meetings have been uploaded into the calendar.
    """

    topic = "fedocal.calendar.upload"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "Schema for messages sent when meetings have been "
        "uploaded into the calendar",
        "type": "object",
        "properties": {"agent": {"type": "string"}, "calendar": CALENDAR},
        "required": ["calendar", "agent"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "{user} has uploaded meetings in the calendar {calendar}".format(
            user=self.body["agent"],
            calendar=self.body["calendar"]["calendar_name"],
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return str(self)


class CalendarDeleteV1(FedocalMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by fedocal when a calendar is deleted.
    """

    topic = "fedocal.calendar.delete"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "Schema for messages sent when a calendar is deleted",
        "type": "object",
        "properties": {"agent": {"type": "string"}, "calendar": CALENDAR},
        "required": ["calendar", "agent"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "{user} has deleted the calendar {calendar}".format(
            user=self.body["agent"],
            calendar=self.body["calendar"]["calendar_name"],
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return str(self)


class CalendarClearV1(FedocalMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by fedocal when a calendar is cleared.
    """

    topic = "fedocal.calendar.clear"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "Schema for messages sent when a calendar is cleared",
        "type": "object",
        "properties": {"agent": {"type": "string"}, "calendar": CALENDAR},
        "required": ["calendar", "agent"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "{user} has cleared the calendar {calendar}".format(
            user=self.body["agent"],
            calendar=self.body["calendar"]["calendar_name"],
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return str(self)


class MeetingNewV1(FedocalMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by fedocal when a meeting is created.
    """

    topic = "fedocal.meeting.new"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "Schema for messages sent when a meeting is created",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "calendar": CALENDAR,
            "meeting": MEETING,
        },
        "required": ["agent", "calendar", "meeting"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return (
            "{user} has created a new meeting {meeting} in calendar"
            " {calendar}".format(
                user=self.body["agent"],
                meeting=self.body["meeting"]["meeting_name"],
                calendar=self.body["calendar"]["calendar_name"],
            )
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return str(self)


class MeetingUpdateV1(FedocalMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by fedocal when a meeting is updated.
    """

    topic = "fedocal.meeting.update"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "Schema for messages sent when a meeting is updated",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "calendar": CALENDAR,
            "meeting": MEETING,
        },
        "required": ["agent", "calendar", "meeting"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return "{user} has updated meeting {meeting} in calendar" " {calendar}".format(
            user=self.body["agent"],
            meeting=self.body["meeting"]["meeting_name"],
            calendar=self.body["calendar"]["calendar_name"],
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return str(self)


class MeetingDeleteV1(FedocalMessage):
    """
    A sub-class of a Fedora message that defines a message schema for messages
    published by fedocal when a meeting is deleted.
    """

    topic = "fedocal.meeting.delete"

    body_schema = {
        "id": SCHEMA_URL + topic,
        "$schema": "http://json-schema.org/draft-04/schema#",
        "description": "Schema for messages sent when a meeting is deleted",
        "type": "object",
        "properties": {
            "agent": {"type": "string"},
            "calendar": CALENDAR,
            "meeting": MEETING,
        },
        "required": ["agent", "calendar", "meeting"],
    }

    def __str__(self):
        """Return a complete human-readable representation of the message."""
        return (
            "{user} has deleted the meeting {meeting} from the calendar"
            " {calendar}".format(
                user=self.body["agent"],
                meeting=self.body["meeting"]["meeting_name"],
                calendar=self.body["calendar"]["calendar_name"],
            )
        )

    @property
    def summary(self):
        """Return a summary of the message."""
        return str(self)



# Copyright (C) 2020  Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from fedora_messaging import message

import pkg_resources

from .messages import (  # noqa: F401
    CalendarClearV1,
    CalendarDeleteV1,
    CalendarNewV1,
    CalendarUpdateV1,
    CalendarUploadV1,
    MeetingDeleteV1,
    MeetingNewV1,
    MeetingUpdateV1,
    ReminderV1,
)


def get_message_object_from_topic(topic):
    """Returns the Message class corresponding to the topic."""

    output = None

    for entry_point in pkg_resources.iter_entry_points("fedora.messages"):
        cls = entry_point.load()
        if cls().topic == topic:
            output = cls
            break

    if output is None:
        output = message.Message

    return output



# Copyright (C) 2020  Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""Utilities for the unit tests."""


DUMMY_CALENDAR = {
    "calendar_name": "test_calendar",
    "calendar_contact": "foo@example.com",
    "calendar_description": "A test calendar",
    "calendar_editor_group": None,
    "calendar_admin_group": "syadmin",
    "calendar_status": "active",
}

DUMMY_MEETING = {
    "meeting_time_start": "12:00:00",
    "meeting_name": "wat",
    "meeting_id": 42,
    "meeting_time_stop": "12:00:00",
    "calendar_name": "awesome",
    "meeting_location": None,
    "meeting_date_end": "2013-09-21",
    "meeting_timezone": "UTC",
    "meeting_manager": ["ralph"],
    "meeting_date": "2013-09-20",
    "meeting_information": "awesome",
    "meeting_region": None,
}



# Copyright (C) 2020  Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""Unit tests for common properties of the message schemas."""

from .utils import DUMMY_CALENDAR, DUMMY_MEETING
from ..messages import ReminderV1


def test_properties():
    """Assert some properties are correct."""
    body = {"meeting": DUMMY_MEETING, "calendar": DUMMY_CALENDAR, "agent": "dummy_user"}
    message = ReminderV1(body=body)

    assert message.app_name == "fedocal"
    assert message.app_icon == "https://apps.fedoraproject.org/img/icons/fedocal.png"
    assert message.agent == "dummy_user"
    assert (
        message.agent_avatar == "https://seccdn.libravatar.org/avatar/"
        "b0ec8f140ba7eee8f4bfc7955858bc43c14fdd83edadf7b3819459efcaa5c690?s=64&d=retro"
    )
    assert message.usernames == ["dummy_user"]



# Copyright (C) 2020  Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""Unit tests for the message schema."""

import datetime

from jsonschema import ValidationError

import pytest

from .utils import DUMMY_CALENDAR, DUMMY_MEETING
from ..messages import ReminderV1


def test_minimal():
    """
    Assert the message schema validates a message with the required fields.
    """
    body = {
        "meeting": DUMMY_MEETING,
        "calendar": DUMMY_CALENDAR,
    }
    message = ReminderV1(body=body)
    message.validate()
    assert message.url is None


def test_full():
    """
    Assert the message schema validates a message with the required fields.
    """
    body = {
        "meeting": DUMMY_MEETING,
        "calendar": DUMMY_CALENDAR,
    }
    message = ReminderV1(body=body)
    message.validate()
    assert message.url is None


def test_missing_fields():
    """Assert an exception is actually raised on validation failure."""
    minimal_message = {
        "calendar": DUMMY_CALENDAR,
    }
    message = ReminderV1(body=minimal_message)
    with pytest.raises(ValidationError):
        message.validate()


def test_str_now():
    """Assert __str__ produces a human-readable message."""
    meeting = DUMMY_MEETING.copy()
    meeting["meeting_date"] = (datetime.datetime.utcnow()).strftime("%Y-%m-%d")
    meeting["meeting_time_start"] = (datetime.datetime.utcnow()).strftime("%H:%M:%S")
    body = {
        "meeting": meeting,
        "calendar": DUMMY_CALENDAR,
    }
    expected_str = (
        "Friendly reminder!  The 'wat' meeting from the "
        "'test_calendar' calendar starts right now"
    )
    message = ReminderV1(body=body)
    message.validate()
    assert expected_str == str(message)


def test_str_thirty_minutes():
    """Assert __str__ produces a human-readable message."""
    meeting = DUMMY_MEETING.copy()
    meeting["meeting_date"] = (
        datetime.datetime.utcnow() + datetime.timedelta(minutes=30)
    ).strftime("%Y-%m-%d")
    meeting["meeting_time_start"] = (
        datetime.datetime.utcnow() + datetime.timedelta(minutes=30)
    ).strftime("%H:%M:%S")
    body = {
        "meeting": meeting,
        "calendar": DUMMY_CALENDAR,
    }
    expected_str = (
        "Friendly reminder!  The 'wat' meeting from the "
        "'test_calendar' calendar starts in 29 minutes"
    )
    message = ReminderV1(body=body)
    message.validate()
    assert expected_str == str(message)


def test_str_two_hours():
    """Assert __str__ produces a human-readable message."""
    meeting = DUMMY_MEETING.copy()
    meeting["meeting_date"] = (
        datetime.datetime.utcnow() + datetime.timedelta(hours=2)
    ).strftime("%Y-%m-%d")
    meeting["meeting_time_start"] = (
        datetime.datetime.utcnow() + datetime.timedelta(hours=2)
    ).strftime("%H:%M:%S")
    body = {
        "meeting": meeting,
        "calendar": DUMMY_CALENDAR,
    }
    expected_str = (
        "Friendly reminder!  The 'wat' meeting from the "
        "'test_calendar' calendar starts in 1 hour"
    )
    message = ReminderV1(body=body)
    message.validate()
    assert expected_str == str(message)


def test_str_five_days():
    """Assert __str__ produces a human-readable message."""
    meeting = DUMMY_MEETING.copy()
    meeting["meeting_date"] = (
        datetime.datetime.utcnow() + datetime.timedelta(days=5)
    ).strftime("%Y-%m-%d")
    meeting["meeting_time_start"] = (
        datetime.datetime.utcnow() + datetime.timedelta(days=5)
    ).strftime("%H:%M:%S")
    body = {
        "meeting": meeting,
        "calendar": DUMMY_CALENDAR,
    }
    expected_str = (
        "Friendly reminder!  The 'wat' meeting from the "
        "'test_calendar' calendar starts in 4 days"
    )
    message = ReminderV1(body=body)
    message.validate()
    assert expected_str == str(message)


def test_summary():
    """Assert the summary is correct."""
    body = {
        "meeting": DUMMY_MEETING,
        "calendar": DUMMY_CALENDAR,
    }
    expected_summary = (
        "Remember about the meeting 'wat' from the 'test_calendar' calendar"
    )
    message = ReminderV1(body=body)
    assert expected_summary == message.summary
